
#include <math.h>
#include <stdio.h>
//#include <conio.h>
//#include <process.h>
#include <string.h>
#include <stdlib.h>


#include <QString>
#include <QFile>
#include <QFileDialog>
#include <QStringList>
#include <QInputDialog>

#include "dialog_parameters.h"
#include "mainwindow.h"


#include "fldl54.h"

#define EXTERN
#include "glob_var.h"


#include "iodb.h"


static QString fileNamePath ;

int InitAllWorkDataFiles() ;
void CloseAllFiles() ;

Dialog_Parameters::Dialog_Parameters(QWidget *parent) :
    QDialog(parent){
    setupUi(this);

    int i ;
    for(i=0;i<MOLECULE_TYPES_NUMBER;i++) comboBox_Symmetry->addItem(  QString(mol_types[i]) );
    for(i=0;i<HAMILTONIANS_TYPES_NUMBER;i++)  comboBox_Hamiltonian->addItem(  QString(Models[i]) );
    for(i=0;i<MEASUREMENTS_TYPES_NUMBER;i++)  comboBox_Measured->addItem(  QString(MeasurementsType[i]) );

    spinBox_NVA->setMaximum(100) ;
    spinBox_NVB->setMaximum(100) ;
}


static int iDipMomFile ;

void Dialog_Parameters::InitDialog()
{
    int LoadDipolMomentParameters(char *name) ;

    lineEdit_SpectrumName->setText(QString(WorkSpace.name[1]));
    lineEdit_UpperSateParamsName->setText(QString(WorkSpace.name[2]));
    lineEdit_LowStateParamsName->setText(QString(WorkSpace.name[3]));
    lineEdit_DipMomentParamsName->setText(QString(WorkSpace.name[4]));
    lineEdit_UpperStateBaseName->setText(QString(WorkSpace.name[5]));
    lineEdit_IdentificationDataName->setText(QString(WorkSpace.name[6]));
    lineEdit_RecogParamsName->setText(QString(WorkSpace.name[7]));
    lineEdit_LowerVibStateName->setText(QString(WorkSpace.name[8]));

    this->checkBox_Prediction->setChecked( WorkSpace.iUsePrediction ) ;
    this->checkBox_ExperienceLowLevels->setChecked( WorkSpace.iUseExperienceLowLevels ) ;

    iDipMomFile = 0 ;
    if(strlen(WorkSpace.name[4])!=0) iDipMomFile = LoadDipolMomentParameters( WorkSpace.name[4] ) ;

    comboBox_Symmetry->setCurrentIndex( WorkSpace.iSymType );
    spinBox_NVA->setValue( WorkSpace.nva );
    spinBox_NVB->setValue( WorkSpace.nvb );
    comboBox_Hamiltonian->setCurrentIndex( WorkSpace.model ) ;
    comboBox_Measured->setCurrentIndex( WorkSpace.iatr );

    spinBox_DiagolalParNumber->setValue( WorkSpace.iNDiagolalPar ) ;
    spinBox_ResonanceParNumber->setValue( WorkSpace.iNResonancePar ) ;

    doubleSpinBox_Path->setValue( WorkSpace.dPathLen ) ;
    doubleSpinBox_Pressure->setValue( WorkSpace.dPressure ) ;
    doubleSpinBox_Temperature->setValue( WorkSpace.dTemperature ) ;
    doubleSpinBox_AlphaD->setValue( WorkSpace.dAlphaD ) ;
    doubleSpinBox_Mass->setValue( WorkSpace.dMassOfMolecule ) ;
    doubleSpinBox_StatSumm->setValue( WorkSpace.dStatSumm ) ;
    doubleSpinBox_Gt->setValue( WorkSpace.dGt ) ;
    doubleSpinBox_Gt1->setValue( WorkSpace.dGt1 ) ;
    doubleSpinBox_Low->setValue( WorkSpace.dLow ) ;
    doubleSpinBox_Up->setValue( WorkSpace.dUp ) ;
    doubleSpinBox_Br->setValue( WorkSpace.dBr ) ;

    checkBox_ExperienceLowLevels->setChecked( WorkSpace.iUseExperienceLowLevels ) ;
    checkBox_LowLevelCombination->setChecked( WorkSpace.LowLevelsCombDiff );
    checkBox_AskOnOverwrite->setChecked( WorkSpace.AskOverwriteMode );
    spinBox_SingleLinesViewNumber->setValue( WorkSpace.iSingleLinesViewNumber );

    PreditionDescroption = WorkSpace.PreditionDescroption ;
    InitPredictionFormatEditor() ;
}



void Dialog_Parameters::InitPredictionFormatEditor()
{

    spinBox_LineFrequensyColumn->setValue( PreditionDescroption.iPrediction_Frequensy_Column+1 ) ;
    spinBox_LineIntensityColumn->setValue( PreditionDescroption.iPrediction_Intensity_Column+1 ) ;
    spinBox_LowerVibrationStateNameColumn->setValue( PreditionDescroption.iPrediction_LowerVibState_Column+1 ) ;
    spinBox_UpperVibrationStateNameColumn->setValue( PreditionDescroption.iPrediction_UpperVibState_Column+1 ) ;

    spinBox_RotationQuantumNumberQuantity->setValue( PreditionDescroption.iPrediction_RotationState_ColumnsNumber ) ;
    iUpperRotationQuantumNumberIndex = 0 ;
    iLowerRotationQuantumNumberIndex = 0 ;
    spinBox_UpperRotationQuantumNumberIndex->setValue( iUpperRotationQuantumNumberIndex+1 );
    spinBox_UpperRotationStatePartColumn->setValue( PreditionDescroption.iPrediction_UpperRotationState_Column[iUpperRotationQuantumNumberIndex]+1 ) ;
    spinBox_LowerRotationQuantumNumberIndex->setValue( iLowerRotationQuantumNumberIndex+1 );
    spinBox_LowerRotationStatePartColumn->setValue( PreditionDescroption.iPrediction_LowerRotationState_Column[iLowerRotationQuantumNumberIndex]+1 ) ;

    spinBox_LowerRotationQuantumNumberIndex->setMaximum( PreditionDescroption.iPrediction_RotationState_ColumnsNumber );
    spinBox_UpperRotationQuantumNumberIndex->setMaximum( PreditionDescroption.iPrediction_RotationState_ColumnsNumber );
    spinBox_LowerEnergyColumn->setValue( PreditionDescroption.iPrediction_LowerEnergyLevels_Column + 1 ) ;

    sprintf(Abuff,"%.3e", PreditionDescroption.dIntensityFactor ) ;
    lineEdit_IntensityMultyplifier->setText(Abuff) ;
}



void Dialog_Parameters::on_buttonBox_Parameters_accepted()
{
    int SaveDipolMomentParameters(char *name) ;
    void SaveWorkSpace(const char *name) ;

    strcpy(WorkSpace.name[1], lineEdit_SpectrumName->text().toLocal8Bit());
    strcpy(WorkSpace.name[2], lineEdit_UpperSateParamsName->text().toLocal8Bit());
    strcpy(WorkSpace.name[3], lineEdit_LowStateParamsName->text().toLocal8Bit());
    strcpy(WorkSpace.name[4], lineEdit_DipMomentParamsName->text().toLocal8Bit());
    strcpy(WorkSpace.name[5], lineEdit_UpperStateBaseName->text().toLocal8Bit());
    strcpy(WorkSpace.name[6], lineEdit_IdentificationDataName->text().toLocal8Bit());
    strcpy(WorkSpace.name[7], lineEdit_RecogParamsName->text().toLocal8Bit());
    strcpy(WorkSpace.name[8], lineEdit_LowerVibStateName->text().toLocal8Bit());

    WorkSpace.iUsePrediction = this->checkBox_Prediction->isChecked() ;
    WorkSpace.iUseExperienceLowLevels = this->checkBox_ExperienceLowLevels->isChecked() ;

    WorkSpace.iSymType = comboBox_Symmetry->currentIndex();
    WorkSpace.nva = spinBox_NVA->value() ;
    WorkSpace.nvb = spinBox_NVB->value() ;
    WorkSpace.model = comboBox_Hamiltonian->currentIndex() ;
    WorkSpace.iatr = comboBox_Measured->currentIndex() ;

    WorkSpace.iNDiagolalPar = spinBox_DiagolalParNumber->value() ;
    WorkSpace.iNResonancePar = spinBox_ResonanceParNumber->value() ;

    WorkSpace.dPathLen = doubleSpinBox_Path->value() ;
    WorkSpace.dPressure = doubleSpinBox_Pressure->value() ;
    WorkSpace.dTemperature = doubleSpinBox_Temperature->value() ;
    WorkSpace.dAlphaD  = doubleSpinBox_AlphaD->value() ;
    WorkSpace.dMassOfMolecule = doubleSpinBox_Mass->value() ;
    WorkSpace.dStatSumm = doubleSpinBox_StatSumm->value() ;
    WorkSpace.dGt = doubleSpinBox_Gt->value() ;
    WorkSpace.dGt1 = doubleSpinBox_Gt1->value() ;
    WorkSpace.dLow = doubleSpinBox_Low->value() ;
    WorkSpace.dUp = doubleSpinBox_Up->value() ;
    WorkSpace.dBr = doubleSpinBox_Br->value() ;

    WorkSpace.iUseExperienceLowLevels = checkBox_ExperienceLowLevels->isChecked() ;
    WorkSpace.LowLevelsCombDiff = checkBox_LowLevelCombination->isChecked() ;
    WorkSpace.AskOverwriteMode = checkBox_AskOnOverwrite->isChecked() ;
    WorkSpace.iSingleLinesViewNumber = spinBox_SingleLinesViewNumber->value() ;

    if( strlen(WorkSpace.name[4])!= 0 ) SaveDipolMomentParameters( WorkSpace.name[4] ) ;

    WorkSpace.PreditionDescroption = PreditionDescroption ;

    //LoadInitiaDataFromEnergyBank() ;
    CloseAllFiles() ;
    SaveWorkSpace("task.wsp") ;
    InitAllWorkDataFiles() ;
    iUpdateMainHeader = 1 ;
}




void Dialog_Parameters::on_toolButton_SpectrumSelect_clicked()
{
    QString Filters = QString("Spectrum file *.spe (*.spe)") ;
    QString FilterRet ;

    QFileDialog myFileDialog(this, tr("Open File"),  QString()) ;

    if( !fileNamePath.isEmpty() ) myFileDialog.setDirectory(QDir(fileNamePath));

    QString fileName = myFileDialog.getOpenFileName (this, tr("Select spectrum file"), QString(""), Filters, &FilterRet, 0  ) ;

     if( !fileName.isEmpty() )
     {
         fileNamePath = fileName.left(fileName.lastIndexOf('/') ) ;

         strcpy(WorkSpace.name[1], fileName.toAscii().constData()) ;
         lineEdit_SpectrumName->setText(QString(WorkSpace.name[1]));
     }
}

void Dialog_Parameters::on_toolButton_UpperStateFileSelect_clicked()
{
    QString Filters = QString("Upper state parameters file *.dat (*.dat)") ;
    QString FilterRet ;

    QFileDialog myFileDialog(this, tr("Open File"),  QString()) ;

    if( !fileNamePath.isEmpty() ) myFileDialog.setDirectory(QDir(fileNamePath));

    QString fileName = myFileDialog.getOpenFileName (this, tr("Select Upper state parameters file"), QString(""), Filters, &FilterRet, 0  ) ;

     if( !fileName.isEmpty() )
     {
         fileNamePath = fileName.left(fileName.lastIndexOf('/') ) ;

         strcpy(WorkSpace.name[2], fileName.toAscii().constData()) ;
         lineEdit_UpperSateParamsName->setText(QString(WorkSpace.name[2]));
     }
}

void Dialog_Parameters::on_toolButton_LowStateFileSelect_clicked()
{
    QString Filters = QString("Lower state parameters file *.par (*.par)") ;
    QString FilterRet ;

    QFileDialog myFileDialog(this, tr("Open File"),  QString()) ;

    if( !fileNamePath.isEmpty() ) myFileDialog.setDirectory(QDir(fileNamePath));

    QString fileName = myFileDialog.getOpenFileName (this, tr("Select Lower state parameters file"), QString(""), Filters, &FilterRet, 0  ) ;

     if( !fileName.isEmpty() )
     {
         fileNamePath = fileName.left(fileName.lastIndexOf('/') ) ;

         strcpy(WorkSpace.name[DEF_LOWER_STATE_PAR_INDEX], fileName.toAscii().constData()) ;
         lineEdit_LowStateParamsName->setText(QString(WorkSpace.name[3]));
     }
}

void Dialog_Parameters::on_toolButton_DipMomentSelect_clicked()
{
    QString Filters = QString("Dipol moment parameters file *.par (*.par)") ;
    QString FilterRet ;

    QFileDialog myFileDialog(this, tr("Open File"),  QString()) ;

    if( !fileNamePath.isEmpty() ) myFileDialog.setDirectory(QDir(fileNamePath));

    QString fileName = myFileDialog.getOpenFileName (this, tr("Select Dipol moment parameters file"), QString(""), Filters, &FilterRet, 0  ) ;

     if( !fileName.isEmpty() )
     {
         fileNamePath = fileName.left(fileName.lastIndexOf('/') ) ;

         strcpy(WorkSpace.name[4], fileName.toAscii().constData()) ;
         lineEdit_DipMomentParamsName->setText(QString(WorkSpace.name[4]));
         int LoadDipolMomentParameters(char *name) ;

         LoadDipolMomentParameters(WorkSpace.name[4]) ;
         InitDialog() ;
     }
}

void Dialog_Parameters::on_toolButton_UpperStateBaseSelection_clicked()
{
    QString Filters = QString("Database files *.db7 (*.db7)") ;
    QString FilterRet ;

    QFileDialog myFileDialog(this, tr("Open File"),  QString()) ;

    if( !fileNamePath.isEmpty() ) myFileDialog.setDirectory(QDir(fileNamePath));

    QString fileName = myFileDialog.getOpenFileName (this, tr("Select Upper Energy Levels database file"), QString(""), Filters, &FilterRet, 0  ) ;

     if( !fileName.isEmpty() )
     {
         fileNamePath = fileName.left(fileName.lastIndexOf('/') ) ;

         strcpy(WorkSpace.name[5], fileName.toAscii().constData()) ;
         lineEdit_UpperStateBaseName->setText(QString(WorkSpace.name[5]));
     }
}

void Dialog_Parameters::on_toolButton_IdentificationBaseSelection_clicked()
{
    QString Filters = QString("Database files *.db7 (*.db7)") ;
    QString FilterRet ;

    QFileDialog myFileDialog(this, tr("Open File"),  QString()) ;

    if( !fileNamePath.isEmpty() ) myFileDialog.setDirectory(QDir(fileNamePath));

    QString fileName = myFileDialog.getOpenFileName (this, tr("Select Identification database file"), QString(""), Filters, &FilterRet, 0  ) ;

     if( !fileName.isEmpty() )
     {
         fileNamePath = fileName.left(fileName.lastIndexOf('/') ) ;

         strcpy(WorkSpace.name[6], fileName.toAscii().constData()) ;
         lineEdit_IdentificationDataName->setText(QString(WorkSpace.name[6]));
     }
}

void Dialog_Parameters::on_toolButton_RecofnitionFileSelect_clicked()
{
    QString Filters = QString("Recognition parameters files *.sgn (*.sgn)") ;
    QString FilterRet ;

    QFileDialog myFileDialog(this, tr("Open File"),  QString()) ;

    if( !fileNamePath.isEmpty() ) myFileDialog.setDirectory(QDir(fileNamePath));

    QString fileName = myFileDialog.getSaveFileName (this, tr("Select Recognition parameters file"), QString(""), Filters, &FilterRet, 0  ) ;

     if( !fileName.isEmpty() )
     {
         fileNamePath = fileName.left(fileName.lastIndexOf('/') ) ;

         strcpy(WorkSpace.name[7], fileName.toAscii().constData()) ;
         lineEdit_RecogParamsName->setText(QString(WorkSpace.name[7]));
     }
}

void Dialog_Parameters::on_Dialog_Parameters_finished(int result)
{
}

void Dialog_Parameters::on_Dialog_Parameters_rejected()
{
    //InitAllWorkDataFiles() ;
}

void Dialog_Parameters::on_pushButton_CreadeLevelsBase_clicked()
{
    QString Filters = QString("Bands names file *.rem (*.rem)") ;
    QString FilterRet ;

    QFileDialog myFileDialog(this, tr("Open File"),  QString()) ;

    if( !fileNamePath.isEmpty() ) myFileDialog.setDirectory(QDir(fileNamePath));

    QString fileName = myFileDialog.getOpenFileName (this, tr("Select bands names file"), QString(""), Filters, &FilterRet, 0  ) ;

     if( !fileName.isEmpty() )
     {
         fileNamePath = fileName.left(fileName.lastIndexOf('/') ) ;

         char cVibStatesNamesFile[512] ;
         strcpy(cVibStatesNamesFile, fileName.toAscii().constData()) ;
         lineEdit_RecogParamsName->setText(QString(WorkSpace.name[7]));

         FILE *fp_comments=fopen(cVibStatesNamesFile,"rt");
         int i=0, cSepSymbolFlag = 0 ;
         char cSymbol, *cRemPrt ;
         cRemPrt=VibStateNames[0];
         cSepSymbolFlag=1;
         while(  (i<40) && (feof(fp_comments)==0) )
         {
            if( fscanf(fp_comments,"%s", VibStateNames[i])>0 ) i++ ;
            /*cSymbol=fgetc(fp_comments);
            switch(cSymbol)
            {
            case 0:
            case 0xffff:
            case ' ':
            case 13:
            case 10:
                       if(cSepSymbolFlag==0)
                       {
                          *cRemPrt=0;
                          printf("\n");
                          printf("[%s] - %d        ",VibStateNames[i],i+1);
                          i++;
                          cRemPrt=VibStateNames[i];
                       }
                       cSepSymbolFlag=1;
                       break;
            default:   *cRemPrt=cSymbol;  cRemPrt++;
                       cSepSymbolFlag=0;
            }*/

         }
         fclose(fp_comments);
         /*if(cSepSymbolFlag==0)
         {
               *cRemPrt=0;
               printf("\n[%s-%d]  %c ",VibStateNames[i],i, cSymbol);
               i++;
         }*/

         lhead.Nbands=i;

         QInputDialog InpD;
         bool ok ;
         int iRet ;

         iRet = InpD.getInt(  this, QString("Input value"), QString("Bands quantity:"), lhead.Nbands, 1, 10000, 1, &ok ) ;
         if(ok)
         {
             lhead.Nbands = iRet ;
         }
         else
             return ;

         iRet = InpD.getInt(  this, QString("Input value"), QString("Maximal J:"), WorkSpace.nj2, 1, 10000, 1, &ok ) ;
         if(ok)
         {
             WorkSpace.nj2 = iRet ;
         }
         else
             return ;

         lhead.NRotationStates = WorkSpace.nj2*WorkSpace.nj2 + WorkSpace.nj2 ;
         lhead.RotationStateSize = sizeof(Level_type)*lhead.Nbands+sizeof(ROTATION_STATE_NAME_TYPE);


         /*****   SELECT  DATA_BASE FILE NAME  *****/

         Filters = QString("DataBase *.db7 (*.db7)") ;


         if( !fileNamePath.isEmpty() )
         {
             myFileDialog.setDirectory(QDir(fileNamePath));
         }


         fileName = myFileDialog.getSaveFileName (this, tr("Create File"), QString(""), Filters, &FilterRet, 0  ) ;

         if( !fileName.isEmpty() )
         {
             fileNamePath = fileName.left(fileName.lastIndexOf('/') ) ;

             strcpy(WorkSpace.name[DEF_UPPER_STATE_BASE_INDEX], fileName.toAscii().constData()) ;
         }
         else
             return ;


         FILE *fp_dest=fopen(WorkSpace.name[DEF_UPPER_STATE_BASE_INDEX],"wb");



         fseek(fp_dest,0,SEEK_SET);
         fwrite(&lhead,sizeof(lhead), 1, fp_dest ) ;
         fwrite(VibStateNames,sizeof(VIBRATION_STATE_NAME_TYPE)*lhead.Nbands, 1, fp_dest) ;



         UpperRotationStateName[0]=0;

         //for(j=0; j<sizeof(Level); j++) (char*)(level)[j]=0;
         level.Energy = 0.0 ;
         level.Nlines = 0 ;
         level.Note[0] = 0x0 ;
         level.RMS = 0.0 ;
         int j ;

         for(i=0; i<lhead.NRotationStates; i++)
            {
                getupperstate(i,UpperRotationStateName);

                fwrite(&UpperRotationStateName,sizeof(ROTATION_STATE_NAME_TYPE),1,fp_dest);


                for(j=0; j<lhead.Nbands; j++)
                    fwrite(&level,sizeof(Level_type),1,fp_dest);
            }

         fclose(fp_dest);

         lhead.EndPosition=ftell(fp_dest);
         printf("EndPosition=%ul  ",lhead.EndPosition);


         fclose(fp_dest);
         this->lineEdit_UpperStateBaseName->setText( WorkSpace.name[DEF_UPPER_STATE_BASE_INDEX] );


     }
}

void Dialog_Parameters::on_pushButton_CreateIdentificationBase_clicked()
{
    QString Filters = QString("Spectrum file *.spe (*.spe)") ;
    QString FilterRet ;

    QFileDialog myFileDialog(this, tr("Open File"),  QString()) ;

    if( !fileNamePath.isEmpty() ) myFileDialog.setDirectory(QDir(fileNamePath));

    QString fileName = myFileDialog.getOpenFileName (this, tr("Select spectrum file"), QString(""), Filters, &FilterRet, 0  ) ;

    char soursename[256], destname[256] ;

     if( !fileName.isEmpty() )
     {
         fileNamePath = fileName.left(fileName.lastIndexOf('/') ) ;

         strcpy( soursename, fileName.toAscii().constData()) ;
         strcpy( destname, soursename ) ;
         int i,l ;
         l = strlen(destname) ;
         i = l ;
         char *cPtr = &destname[l] ;
         while((i>0)&&(*cPtr!='.'))
         {
             cPtr-- ;
             i-- ;
         }
         if(i>0) *cPtr = 0x0 ;
         strcat( destname, ".db7" ) ;
     }
     else
         return ;

     /*****   SELECT  DATA_BASE FILE NAME  *****/

     Filters = QString("DataBase *.db7 (*.db7)") ;


     if( !fileNamePath.isEmpty() )
     {
         myFileDialog.setDirectory(QDir(fileNamePath));
     }


     fileName = myFileDialog.getSaveFileName (this, tr("Create File"), QString(destname), Filters, &FilterRet, 0  ) ;

     if( !fileName.isEmpty() )
     {
         fileNamePath = fileName.left(fileName.lastIndexOf('/') ) ;

         strcpy(WorkSpace.name[DEF_LINES_ASSIGN_BASE_INDEX], fileName.toAscii().constData()) ;
     }
     else
         return ;


     FILE *fp_dest=fopen(WorkSpace.name[DEF_LINES_ASSIGN_BASE_INDEX],"wb");

     if( fp_dest == NULL ) return ;

     FILE *fp_source = fopen(soursename,"rt") ;
     if( fp_source == NULL ) return ;



     int i=0,  j=0,   Size=sizeof(Line_type) ;
     int i1=0, iTmp ;
     char br[256], bspectrum[30], bstrength[30] ;
     double dTmp0, dTmp1, dTmp2 ;
     Line_type sline, line2 ;
     LinesHead_type shead ;

     do{
         fgets(Abuff,80,fp_source);  br[0]=0;
         //       puts(Abuff); printf(" %d",i);
         if (  (iTmp=sscanf(Abuff,"%s %s   %s  ",bspectrum, bstrength, br)) > 0  )
         {
               sline.Frequansy  = /*atof(bspectrum)*/QString(bspectrum).toDouble();
               dTmp1  = /*atof(bstrength)*/QString(bstrength).toDouble();
               dTmp2  = atof(br);
               if( fabs(dTmp1)>1.0e-18 )
                    sline.Intensity = dTmp1;
               else
               {
                    //gotoxy(1,20);
                    printf("Warning: line have very little intensity (absorption)");
                    printf("\n%f  %f  %f",sline.Frequansy,dTmp1 , dTmp2);
                    //getch();
                    sline.Intensity = 0.0;
                }

                dTmp0=sline.Frequansy;
                if(iTmp<3) sline.MissError  = 0.0;
                else
                {
                     if(fabs(dTmp2)>1.0e-18)
                         sline.MissError  = dTmp2;
                     else
                     {
                          //gotoxy(1,20);
                          printf("Warning: line have very little measure error");
                          printf("\n%f  %f  %f",sline.Frequansy,dTmp1 , dTmp2);
                          //getch();
                          sline.MissError = 0.0;
                     }
                }

                if(dTmp0>sline.Frequansy)
                {
                      //gotoxy(1,20);
                      printf("Warning: invertion of order of frequencies ");
                      printf("\n%f  %f  %f",sline.Frequansy,dTmp1 , dTmp2);
                      //getch();
                }

                if(iTmp<3) sline.MissError  = 0.0;



                dTmp0=sline.Frequansy;




                for(j=0;j<DEF_ASSIGNMENT_FIELDS_NUMBER; j++)
                {
                    sline.V_upper[j][0]=0;
                    sline.V_ground[j][0]=0;
                    sline.upper_state[j][0]  = 0;
                    sline.ground_state[j][0] = 0;
                    sline.Note[j][0]=0;
                }
                sline.del=0;


                int k =WriteLine(fp_dest,i,&sline);

                ReadLine(fp_dest,i,&line2);

                if( (sline.Frequansy != line2.Frequansy)||( k==-1 ) )
                {
                     i1++; printf("\nError of write-- %d ",i1);
                     k =WriteLine(fp_dest,i,&sline);
                              //getch();
                }
                ++ i;
          }

     }while(feof(fp_source)==0);

     fclose(fp_source);

     shead.EndPosition=ftell(fp_dest);
     shead.Number=i;
     shead.LineSize=Size;


     fseek(fp_dest,0,SEEK_SET);
     fwrite(&shead, sizeof(shead), 1, fp_dest );
     printf("Number=%d  Cat.pos.=%d  ",
                          shead.Number, shead.EndPosition);
     printf("u.s.=%d\n", shead.LineSize);

     fclose(fp_dest);

    lineEdit_IdentificationDataName->setText( WorkSpace.name[DEF_LINES_ASSIGN_BASE_INDEX] ) ;
}



void Dialog_Parameters::on_spinBox_LineFrequensyColumn_valueChanged(QString )
{
    PreditionDescroption.iPrediction_Frequensy_Column = spinBox_LineFrequensyColumn->value() - 1 ;
}

void Dialog_Parameters::on_spinBox_LineIntensityColumn_valueChanged(int )
{
    PreditionDescroption.iPrediction_Intensity_Column = spinBox_LineIntensityColumn->value() - 1 ;
}

void Dialog_Parameters::on_spinBox_RotationQuantumNumberQuantity_valueChanged(int )
{
    PreditionDescroption.iPrediction_RotationState_ColumnsNumber = spinBox_RotationQuantumNumberQuantity->value() ;

    this->spinBox_LowerRotationQuantumNumberIndex->setMaximum( PreditionDescroption.iPrediction_RotationState_ColumnsNumber );
    this->spinBox_UpperRotationQuantumNumberIndex->setMaximum( PreditionDescroption.iPrediction_RotationState_ColumnsNumber );
}

void Dialog_Parameters::on_spinBox_UpperRotationQuantumNumberIndex_valueChanged(int )
{
    iUpperRotationQuantumNumberIndex = spinBox_UpperRotationQuantumNumberIndex->value() - 1 ;
    this->spinBox_UpperRotationStatePartColumn->setValue(
            PreditionDescroption.iPrediction_UpperRotationState_Column[iUpperRotationQuantumNumberIndex] + 1 ) ;
}

void Dialog_Parameters::on_spinBox_UpperRotationStatePartColumn_valueChanged(int )
{
    PreditionDescroption.iPrediction_UpperRotationState_Column[iUpperRotationQuantumNumberIndex] = spinBox_UpperRotationStatePartColumn->value() - 1 ;
}

void Dialog_Parameters::on_spinBox_LowerRotationQuantumNumberIndex_valueChanged(int )
{
    iLowerRotationQuantumNumberIndex = spinBox_LowerRotationQuantumNumberIndex->value() - 1 ;
    this->spinBox_LowerRotationStatePartColumn->setValue(
            PreditionDescroption.iPrediction_LowerRotationState_Column[iLowerRotationQuantumNumberIndex] + 1 ) ;

}

void Dialog_Parameters::on_spinBox_LowerRotationStatePartColumn_valueChanged(int )
{
    PreditionDescroption.iPrediction_LowerRotationState_Column[iLowerRotationQuantumNumberIndex] = spinBox_LowerRotationStatePartColumn->value() - 1 ;
}

void Dialog_Parameters::on_spinBox_LowerVibrationStateNameColumn_valueChanged(int )
{
    PreditionDescroption.iPrediction_LowerVibState_Column = spinBox_LowerVibrationStateNameColumn->value() - 1 ;
}

void Dialog_Parameters::on_spinBox_UpperVibrationStateNameColumn_valueChanged(int )
{
    PreditionDescroption.iPrediction_UpperVibState_Column = spinBox_UpperVibrationStateNameColumn->value() - 1 ;
}

void Dialog_Parameters::on_pushButton_SetDefaultsPredictionDescription_clicked()
{
    void InitPredictionFileDescription() ;

    InitPredictionFileDescription() ;

    PreditionDescroption = WorkSpace.PreditionDescroption ;
    InitPredictionFormatEditor() ;
}

void Dialog_Parameters::on_spinBox_LowerEnergyColumn_valueChanged(int )
{
    PreditionDescroption.iPrediction_LowerEnergyLevels_Column = spinBox_LowerEnergyColumn->value() - 1 ;
}

void Dialog_Parameters::on_lineEdit_IntensityMultyplifier_textChanged(QString )
{
    PreditionDescroption.dIntensityFactor = lineEdit_IntensityMultyplifier->text().toDouble() ;
}



